/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 *
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom.components;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import org.json.JSONException;
import org.json.JSONObject;
import org.odftoolkit.odfdom.component.OdfOperationDocument;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.AttributeImpl;
import com.openexchange.office.odf.DLNode;
import com.openexchange.office.odf.Namespaces;
import com.openexchange.office.odt.dom.TextField;

public class TextFieldComponent extends TextSpan_Base {

	public TextFieldComponent(ComponentContext parentContext, DLNode<Object> textFieldNode, int componentNumber) {
		super(parentContext, textFieldNode, componentNumber);
	}

	@Override
	public void applyAttrsFromJSON(OdfOperationDocument operationDocument, JSONObject attrs)
			throws JSONException, SAXException {

		final JSONObject character = attrs.optJSONObject("character");
		if(character!=null) {
			final JSONObject json = character.optJSONObject("field");
			if(json!=null) {
				final HashMap<String, AttributeImpl> attributes = ((TextField)getObject()).getAttributes().getMap();
				final Iterator<Entry<String, Object>> jsonIter = json.entrySet().iterator();
				while(jsonIter.hasNext()) {
					final Entry<String, Object> jsonEntry = jsonIter.next();
					if(jsonEntry.getValue() instanceof String) {
						final String qName = "text:" + jsonEntry.getKey();
						attributes.put(qName, new AttributeImpl(Namespaces.TEXT, qName, jsonEntry.getKey(), (String)jsonEntry.getValue()));
					}
				}
			}
		}
		super.applyAttrsFromJSON(operationDocument, attrs);
	}

	@Override
	public JSONObject createJSONAttrs(OdfOperationDocument operationDocument, JSONObject attrs)
		throws JSONException, SAXException {

		final HashMap<String, AttributeImpl> attributes = ((TextField)getObject()).getAttributes().getMap();
		if(attributes.size()>0) {
			final HashMap<String, String> map = new HashMap<String, String>(attributes.size());
			final Iterator<AttributeImpl> attrIter = attributes.values().iterator();
			while(attrIter.hasNext()) {
				final AttributeImpl attr = attrIter.next();
				map.put(attr.getLocalName(), attr.getValue());
			}
			JSONObject character = attrs.optJSONObject("character");
			if(character==null) {
				character = new JSONObject();
				attrs.put("character", character);
			}
			character.put("field", map);
		}
		return super.createJSONAttrs(operationDocument, attrs);
	}
}
