/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office;

import java.io.InputStream;

import org.json.JSONObject;

import com.openexchange.session.Session;


public interface IPreviewImporter {

	
   /**
    * Creates the preview and global operations to be used for a preview of the
    * current sheet/page. The document model that is used by the filter is stored
    * in the documentProperties (PROP_XLSX_DOCUMENT), this property is necessary
    * for further createPreview operations.
    *
    * @param session
    *  The session of the client requesting the preview operations.
    *
    * @param inputDocument
    *  The document stream.
    *
    * @param documentProperties
    *  Document properties which provides/receives document specific properties
    *  like language, active sheet/page.
    *
    */
    public void createDocumentModel(Session session, InputStream inputDocument, DocumentProperties documentProperties)
		throws FilterException;

   /**
    * Creates the preview and global operations to be used for a preview of the
    * current sheet/page.
    *
    * @param documentProperties
    *  Document properties which provides/receives document specific properties
    *  like language, active sheet/page.
    *
    * @return
    *  The JSON Object containing preview & global operations from the given document
    *  
    *  The active sheet index is saved as Integer in the documentProperties (PROP_XLSX_DOCUMENT)
    */
   public JSONObject createPreviewOperations(DocumentProperties documentProperties)
       throws FilterException;

   /**
    * Creates the remaining operations of the document referenced by
    * createPreviewOperations. Means that createOperations cannot be called
    * without createPreviewOperations.
    *
    * @param documentProperties
    *  Document properties which provides/receives document specific properties
    *  like language, active sheet/page.
    *
    * @return
    *  The JSON Object containing preview & global operations from the given document
    */
   public JSONObject createOutstandingOperations(DocumentProperties documentProperties)
       throws FilterException;
}
